package ru.dbotthepony.kommons.io

import java.io.InputStream
import java.io.OutputStream

class ByteKey(private vararg val bytes: Byte) : Comparable<ByteKey> {
	constructor(key: String) : this(*key.toByteArray())

	override fun equals(other: Any?): Boolean {
		return this === other || other is ByteKey && other.bytes.contentEquals(bytes)
	}

	val size: Int
		get() = bytes.size

	operator fun get(index: Int): Byte {
		return bytes[index]
	}

	fun write(stream: OutputStream) {
		stream.writeVarInt(bytes.size)
		stream.write(bytes)
	}

	fun writeRaw(stream: OutputStream) {
		stream.write(bytes)
	}

	fun toByteArray(): ByteArray {
		return bytes.clone()
	}

	override fun hashCode(): Int {
		return bytes.contentHashCode()
	}

	override fun toString(): String {
		return "ByteKey[${bytes.map { it.toInt() and 0xFF }.joinToString(", ")}]"
	}

	override fun compareTo(other: ByteKey): Int {
		val cmp = size.compareTo(other.size)
		if (cmp != 0) return cmp

		for (i in bytes.indices) {
			if (bytes[i].toInt() and 0xFF > other.bytes[i].toInt() and 0xFF) {
				return 1
			} else if (bytes[i].toInt() and 0xFF < other.bytes[i].toInt() and 0xFF) {
				return -1
			}
		}

		return 0
	}
}
